@@ -49,6 +49,13 @@
 
 #include "d4lib.h"
 
+#ifdef MACOS
+#define read(a,b,c)  mRead(a,b,c)
+#define write(a,b,c) mWrite(a,b,c)
+#else
+#define read(a,b,c)  devRead(a,b,c)
+#endif
+
 #ifndef RDTIMEOUT
 #define RDTIMEOUT 2000
 #define WRTIMEOUT 2000
@@ -57,7 +64,7 @@
 int d4WrTimeout = WRTIMEOUT;
 int d4RdTimeout = RDTIMEOUT;
 int ppid        = 0;
-
+extern int doRw;
 int debugD4     = 0;
 
 typedef void (*signalHandler_t)(int);
@@ -166,6 +173,7 @@ static errorMessage_t errorMessage[] =
                               memset((void*)&ti,0,sizeof(ti)); \
                               memset((void*)&oti,0,sizeof(oti)); \
                               setitimer( ITIMER_REAL ,&ti, &oti); \
+                              doRw = 0; \
                             }
 
 #define SET_TIMER(ti,oti,val) { memset((void*)&ti,0,sizeof(ti)); \
@@ -174,6 +182,7 @@ static errorMessage_t errorMessage[] =
                                 ti.it_value.tv_usec = (val%1000)*1000; \
                                 setitimer( ITIMER_REAL ,&ti, &oti); \
                                 sig = signal(SIGALRM, sigAlarm); \
+                                doRw = 1; \
                               }
 
 /*******************************************************************/
@@ -210,6 +219,7 @@ void printHexValues(char *dir, uc *buf, int len)
 static void sigAlarm(int code)
 {
    timeoutGot = -1;
+   doRw = 0;
 }
 
 
@@ -290,12 +300,12 @@ static void printCmdType(uc *cmd)
 /*                                                                 */
 /*******************************************************************/
 
-static int writeCmd(int fd, char *cmd, int len)
+static int writeCmd(int fd, unsigned char *cmd, int len)
 {
    int w;
    int i = 0;
    struct  itimerval ti, oti;
-
+   int cnt;
 # if PTIME
    struct timeval beg, end;
    long dt;
@@ -311,8 +321,8 @@ static int writeCmd(int fd, char *cmd, int len)
          printHexValues("Send: ",cmd,len);
       }
       else
-         {
-      printHexValues("Send: ",cmd,6);
+      {
+         printHexValues("Send: ",cmd,6);
       }
    }
 
@@ -321,12 +331,18 @@ static int writeCmd(int fd, char *cmd, int len)
 
    timeoutGot = 0;
    errno = 0;
+   cnt = 0;
    while ( i < len )
    {
       SET_TIMER(ti,oti,d4WrTimeout);         
       w = write(fd, cmd+i,len-i);
       RESET_TIMER(ti,oti);
-      if ( w < 0 )
+      if ( w < 0 && errno == EAGAIN && cnt < 600 )
+      {
+         usleep(100000);
+         cnt++;
+      }
+      else if ( w < 0 )
       {
          if ( debugD4 )
          {
@@ -400,17 +416,17 @@ static int readAnswer(int fd, unsigned char *buf, int len)
          {
             if ( debugD4 )
                fprintf(stderr,"Timeout 1 at readAnswer() rcv %d bytes\n",total);
-	    timeoutGot = 1;
+            timeoutGot = 1;
             break;
          }
-	 count++;
-	 if ( count >= 20 )
-	 {
-	     timeoutGot = 1;
-	     if ( rd == 0 )
-	        errno = -1; /* tell that there is an abnormal coditon */
-	     break;
-	 }
+         count++;
+         if ( count >= 20 )
+         {
+             timeoutGot = 1;
+             if ( rd == 0 )
+                errno = -1; /* tell that there is an abnormal condition */
+             break;
+         }
          errno = 0;
          continue;
       }
@@ -476,7 +492,6 @@ static int _readData(int fd, uc socketID, uc *buf, int len)
 
    /* read the first 6 bytes */
    gettimeofday(&beg, NULL);
-
    while ( total < 6 )
    {
       SET_TIMER(ti,oti, d4RdTimeout);
@@ -539,6 +554,7 @@ static int _readData(int fd, uc socketID, uc *buf, int len)
          printHexValues("Recv: ",buf,total);
       return total;
    }
+
    return -1;
 }
 
@@ -724,7 +740,7 @@ int GetSocketID(int fd, char *serviceName)
    cmd->credit   = 1;
    cmd->control  = 0;
    cmd->command  = 0x09;
-   strcpy(buf+sizeof(cmdHeader_t), serviceName);
+   strcpy((char*)buf+sizeof(cmdHeader_t), serviceName);
 
    rd = sendReceiveCmd(fd, buf, len, rBuf, len+2);
    if ( rd > 0 )
@@ -986,6 +1002,8 @@ int writeData(int fd, uc socketID, uc *buf, int len, int eoj)
    long dt;
    static uc *buffer = NULL;
    static int bLen   = 0;
+   int cnt=0;
+
    if ( debugD4 )
    {
       fprintf(stderr,"--- Send Data      ---\n");
@@ -1018,10 +1036,26 @@ int writeData(int fd, uc socketID, uc *buf, int len, int eoj)
       RESET_TIMER(ti,oti);
       if ( ret == -1 )
       {
-         perror("write: ");
+         /* DEC-10-2005: resource temporary not available, on 2.6 
+          * kernel try again
+          */
+         if ( errno == EAGAIN )
+         {
+            if (cnt < 600 ) /* time out = 1 minute */
+            {
+               ret = 0;
+               cnt++;
+               usleep(100000); /* wait 100 ms */
+            }
+         }
+         else
+         {
+             perror("write");
+         }
       }
       else
       {
+         cnt = 0;
          wr += ret;
       }
    }
@@ -1034,10 +1068,10 @@ int writeData(int fd, uc socketID, uc *buf, int len, int eoj)
       dt += end.tv_usec - beg.tv_usec;
 # endif  
       fprintf(stderr,"Send: ");
-      for ( ret = 0; (wr > 0) && (ret < ((wr > 20) ? 20 : wr)) ; ret++ )
+      for ( ret = 0; (wr > 0) && (ret < ((wr > 50) ? 50 : wr)) ; ret++ )
          fprintf(stderr,"%02x ", buffer[ret]);
       fprintf(stderr,"\n      ");
-      for ( ret = 0; (wr > 0) && (ret < ((wr > 20) ? 20 : wr)) ; ret++ )
+      for ( ret = 0; (wr > 0) && (ret < ((wr > 50) ? 50 : wr)) ; ret++ )
          fprintf(stderr,"%c  ", isprint(buffer[ret])&&!isspace(buffer[ret])?buffer[ret]:' ');
       fprintf(stderr,"\n");
 # if PTIME
