@@ -30,6 +30,10 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
+#ifdef MACOS
+#include "usbHlp.h"
+#endif
+
 #include "mtink.h"
 
 #include "rcfile.h"
@@ -59,8 +63,11 @@ int   py                = 0;
 int   plc               = 0;
 int   plm               = 0;
 int   plb               = 0;
+int   plp               = 0;
+int   plx               = 0;
 char *printerName       = NULL;
 char *printerState;
+int   findDevice        = 0;
 
 configData_t actConfig;
 char *mainResource;
@@ -140,6 +147,9 @@ void setPrinterStateLabel(int state)
 void exit_CB(int val)
 {
    callPrg(TERMINATE,NULL, 0, 0, 0, NULL, NULL);
+#ifdef MACOS
+   unlinkFiles();
+#endif
    exit(val);
 }
 
@@ -160,7 +170,12 @@ void printSyntax(int error, char *lang)
 
    fprintText(out, "%s", lang, "syntax1");
    fprintf(out, "%s\n",prgName);
+#ifdef MACOS
+   fprintText(out, "%s\n", lang, "syntaxM");
+   unlinkFiles();
+#else
    fprintText(out, "%s\n", lang, "syntax2");
+#endif
 
    exit(error);
 }
@@ -179,6 +194,17 @@ void parseArgs(int argc, char **argv, char *lang)
    char *t;
    char  buf[256];
 
+   /* look first for the option -u */
+   for (i = 0; i < argc;i++ )
+   {
+      if ( strncmp("-u", argv[i], 2) == 0 ||
+                strncmp("--u", argv[i], 3) == 0 )
+      {
+          utf8 = 1;
+          lang[2] = '8'; lang[3] = '\0';
+      }
+   }
+
    while ( argc )
    {
       if ( strncmp("-h", argv[0], 2) == 0 ||
@@ -186,6 +212,12 @@ void parseArgs(int argc, char **argv, char *lang)
       {
          printSyntax(0, lang);
       }
+      if ( strncmp("-f", argv[0], 2) == 0 ||
+                strncmp("--f", argv[0], 3) == 0 )
+      {
+         findDevice = 1;;
+      }
+      else
       if ( strncmp("-W", argv[0], 2) == 0 )
       {
          ynFunc = askYn;
@@ -242,7 +274,7 @@ void parseArgs(int argc, char **argv, char *lang)
       else if ( strncmp("-a", argv[0], 2) == 0 ||
                 strncmp("--a", argv[0], 3) == 0 )
       {
-          /* aliggn head command */
+          /* align head command */
            doAlign  = 1;
       }
       else if ( strncmp("-c", argv[0], 2) == 0 ||
@@ -295,7 +327,6 @@ void parseArgs(int argc, char **argv, char *lang)
       else if ( strncmp("-u", argv[0], 2) == 0 ||
                 strncmp("--u", argv[0], 3) == 0 )
       {
-          /* printer status command*/
           utf8 = 1;
           lang[2] = '8'; lang[3] = '\0';
       }
@@ -347,9 +378,10 @@ void parseArgs(int argc, char **argv, char *lang)
       doStatus = 1;
    if ( (doReset + doClean + doCheck + doStatus + doAlign + doExchangeCartridge + doIdent) > 1 )
       printSyntax(1, lang);
+#ifndef MACOS
    if ( device == NULL )
       printSyntax(1,lang);
-
+#endif
 }
 
 
@@ -369,7 +401,9 @@ int main(int argc, char **argv)
    char            printerFound = 0;
    char           *lang         = NULL;
    int             retVal       = 0;
-
+#ifdef MACOS
+   char            prtFile[512];
+#endif
    ynFunc = askYn;
 
    prgName = argv[0];
@@ -392,7 +426,7 @@ int main(int argc, char **argv)
          }
       }
    }
-   
+
    /* if no environment for language we have set en */
 
    if ( strlen(lang) >= 2 )
@@ -421,6 +455,7 @@ int main(int argc, char **argv)
    lang[0] = toupper(s[0]);
    lang[1] = s[1];
    lang[2] = '\0';
+
    if ( utf8 )
    {
       lang[2] = '8';
@@ -440,7 +475,7 @@ int main(int argc, char **argv)
    signal(SIGINT,  sigKill);
    signal(SIGQUIT, sigKill);
    signal(SIGSEGV, sigKill);
-
+#ifndef MACOS
    if ( (model == NULL || doIdent ) && ! d4Printer )
    {
       /* try to detect the printer model */
@@ -454,7 +489,7 @@ int main(int argc, char **argv)
       else if ( state == 0 && retBuf )
       {
          /* may be OK, check for model name */
-         state = decodePrinterType(retBuf, strlen(retBuf));
+         state = decodePrinterType((unsigned char*)retBuf, strlen(retBuf));
          if ( state )
          {
             printerFound = 1;
@@ -480,8 +515,7 @@ int main(int argc, char **argv)
    {
       printerFound = 1;
       printerName = strdup(model);
-   }
-   
+   } 
 
    if ( (printerFound == 0 || printerFound == 2) &&  !d4Printer )
    {
@@ -490,7 +524,7 @@ int main(int argc, char **argv)
       if ( state == 0 && retBuf )
       {
          /* may be OK, check for model name */
-         state = decodePrinterType(retBuf, strlen(retBuf));
+         state = decodePrinterType((unsigned char*)retBuf, strlen(retBuf));
          if ( state )
          {
             printerFound = 1;
@@ -506,8 +540,104 @@ int main(int argc, char **argv)
          /* may be OK and a D4 device */
          printerFound = 0;
       }
-  }
+   }
+#else
+   /* on MacOS we use the lib usb */
+   if ( (state = usbInit()) > 0 )
+   {
+      /* printers detected */
+      if ( createFiles() == 0 )
+      {
+         int i;
+         /* device given, check if OK */
+         if ( device != NULL )
+         {
+            /* look for usb: */
+            s = strstr(device, USB_PRT_NAME_PREFIX);
+            if ( s )
+            {
+               s += strlen(USB_PRT_NAME_PREFIX);
+               if ( isdigit(*s) )
+               {
+                  i = atoi(s);
+               }
+               else /* expect usb:<number>:printer name */
+               {
+                  i = -1;
+               }
 
+               if ( i > -1 && i < state )
+               {
+                  s = strchr(s, ':');
+                  if ( s )
+                  {
+                     s++;
+                     if ( strcmp(s,getPrinterName(i)) )
+                     {
+                        free(device);
+                        device = NULL;
+                     }
+                     else
+                     {
+                        state = i + 1;
+                     }
+                  }
+               }
+               else /* no match */
+               {
+                  free(device);
+                  device = NULL;
+               }
+            }
+            else /* wrong name build */
+            {
+               free(device);
+               device = NULL;
+            }
+         }
+
+         if ( state > 1 && device == NULL && findDevice == 0 )
+         {
+            /* choose printer model */
+            fprintText(stdout, "%s\n", lang,"followingPrintersFound");
+            for ( i = 0; i < state; i++)
+            {
+               printf("%2d: %s\n",i+1, devices[i].prtname);     
+            }
+            fprintText(stdout, "%s\n", lang,"ChoosePrinter");
+            state = askNo(1,state);
+         }
+
+         /* user told to print only device file name */
+         if (findDevice)
+         {
+            for ( i = 0; i < state; i++)
+            {
+               printf(USB_PRT_NAME_PREFIX"%02d:%s\n",i,devices[i].prtname);
+            }
+            exit_CB(0);
+         }
+         
+         /* build device file name */
+         if ((s = getPrinterName(state-1)))
+         {
+            state--;
+            snprintf(prtFile, sizeof(prtFile),USB_PRT_NAME_PREFIX"%02d:%s",state,s);
+            model  = strdup(s);
+            printerName = strdup(s);            
+            device = prtFile;
+            printerFound = 1;
+         }
+      }
+      else
+      {
+         fprintText(stderr,"%s\n",lang, "noEnoughMemory");
+      }
+   }
+   else
+   {
+   }
+#endif
   /* we may have set the -D flag d4Printer in order to avoid detection */
   /* via non D4 protocol. in this case the detection is to be done     */
   /* on the following conditions:
@@ -524,7 +654,7 @@ int main(int argc, char **argv)
       if ( state == 0 && retBuf )
       {
          /* may be OK, check for model name */
-         state = decodePrinterType(retBuf, strlen(retBuf));
+         state = decodePrinterType((unsigned char*)retBuf, strlen(retBuf));
          if ( state )
          {
             printerFound = 1;
@@ -584,14 +714,13 @@ loop:
       callPrg(GET_IQ, actConfig.dev, actConfig.prot, 0, 0, &retBuf, NULL);
       if ( retBuf != NULL && *retBuf )
       {
-         printf("%s:\n", actConfig.name);
          if ( *retBuf == '\0' )
          {
             printerFound = 0;
          }
          else
          {
-            if ( decodeStatus(retBuf, strlen(retBuf), lang) )
+            if ( decodeStatus((unsigned char*)retBuf, strlen(retBuf), lang) )
             {
                printerFound = 1;
                if ( cycle )
@@ -623,7 +752,7 @@ loop:
 
    if ( doCheck )
    {
-      retVal = callPrg(CHECK_NOZZLE, actConfig.dev,actConfig.prot, 0, 0, NULL, NULL);
+      retVal = callPrg(CHECK_NOZZLE, actConfig.dev,actConfig.prot, 0, actConfig.checkNeedReset, NULL, NULL);
    }
 
    if ( doClean )
@@ -689,6 +818,30 @@ int percent(unsigned char *buf)
    return val;
 }
 
+/*******************************************************************/
+/* Function colorType                                              */
+/*                                                                 */
+/* translate the value passed as HEX (1 char) in decimal           */
+/*                                                                 */
+/* Input: unsigned char *buf The hex value to be translated        */
+/*                                                                 */
+/* Return: the decimal value according to hex value                */
+/*                                                                 */
+/*******************************************************************/
+
+int colorType(unsigned char *buf)
+{
+   int val = 0;;
+   if ( buf[0] >= '0' &&  buf[0] <= '9' )
+      val = (buf[0] - '0') * 16;
+   else if ( buf[0] >= 'A' &&  buf[0] <= 'F' )
+      val = (buf[0] - 'A' + 10 ) * 16;
+   else if ( buf[0] >= 'a' &&  buf[0] <= 'f' )
+      val = (buf[0] - 'a' + 10 ) * 16;
+
+   return val;
+}
+
 /*******************************************************************/
 /* Function decodeStatus                                           */
 /*                                                                 */
@@ -718,13 +871,15 @@ int decodeStatus(unsigned char *buf, int len, char *lang)
    plc = 0;
    plm = 0;
    plb = 0;
+   plp = 0;
+   plx = 0;
 
    if ( buf == NULL )
       return infOk;
 
    for ( i = 0; i < len;)
    {
-      if ( strncmp(&buf[i], "ST:", 3 ) == 0 )
+      if ( strncmp((char*)&buf[i], "ST:", 3 ) == 0 )
       {
          infOk = 1;
          i += 3;
@@ -742,7 +897,7 @@ int decodeStatus(unsigned char *buf, int len, char *lang)
          printText("\t%-20s:",lang,"printerState");
          printText(" %s\n\n",lang,printerState);
       }
-      else if ( strncmp(&buf[i], "ER:", 3 ) == 0 )
+      else if ( strncmp((char*)&buf[i], "ER:", 3 ) == 0 )
       {
          i += 3;
          s = NULL;
@@ -764,7 +919,7 @@ int decodeStatus(unsigned char *buf, int len, char *lang)
          }
          i +=3;
       }
-      else if ( strncmp(&buf[i], "IQ:", 3 ) == 0 )
+      else if ( strncmp((char*)&buf[i], "IQ:", 3 ) == 0 )
       {
          infOk = 1;
          i += 3;
@@ -806,6 +961,75 @@ int decodeStatus(unsigned char *buf, int len, char *lang)
             i += 2;
          }
       }
+#if 1
+      else if ( strncmp((char*)&buf[i], "INQ:", 4 ) == 0 )
+      {
+         char colorModel;
+         infOk = 1;
+         i += 4;
+         colIdx = 0;
+         colorModel = buf[i];
+         i++;
+         while ( i < len && colIdx < 8 && buf[i] != ';' )
+         {
+            switch(colIdx)
+            {
+               case 0: pb  = percent(buf+i);
+                  printText("\t%-20s:",lang,"black");
+                  printf(" %2d %%\n",pb);
+               break;
+               case 1: pc  = percent(buf+i);
+                  printText("\t%-20s:",lang,"cyan");
+                  printf(" %2d %%\n",pc);
+               break;
+               case 2: pm  = percent(buf+i);
+                  printText("\t%-20s:",lang,"magenta");
+                  printf(" %2d %%\n",pm);
+               break;
+               case 3: py  = percent(buf+i);
+                  printText("\t%-20s:",lang,"yellow");
+                  printf(" %2d %%\n",py);
+               break;
+               case 4: plc = percent(buf+i);
+                  if ( colorModel == MODEL_KCMycm  ||
+                       colorModel == MODEL_KCMycmY ||
+                       colorModel == MODEL_KCMycmk    )
+                      printText("\t%-20s:",lang,"lcyan");
+                  else
+                      printText("\t%-20s:",lang,"red");
+                  printf(" %2d %%\n",plc);
+               break;
+               case 5: plm = percent(buf+i);
+                  if ( colorModel == MODEL_KCMycm ||
+                       colorModel == MODEL_KCMycmY ||
+                       colorModel == MODEL_KCMycmk    )
+                      printText("\t%-20s:",lang,"lmagenta");
+                  else
+                      printText("\t%-20s:",lang,"blue");
+                  printf(" %2d %%\n",plm);
+               break;
+               case 6: plb = percent(buf+i);
+                  if ( colorModel == MODEL_KCMycmY )
+                      printText("\t%-20s:",lang,"dyellow");
+                  else if ( colorModel == MODEL_KCMycmk )
+                      printText("\t%-20s:",lang,"grey");
+                  else if ( colorModel == MODEL_KCMyRBkX )
+                      printText("\t%-20s:",lang,"lblack");
+                  else if ( colorModel == MODEL_KCMyRBX )
+                      printText("\t%-20s:",lang,"gloss");
+                  printf(" %2d %%\n",plb);
+               break;
+               case 7: pb  = percent(buf+i);
+                  if ( colorModel == MODEL_KCMyRBkX )
+                     printText("\t%-20s:",lang,"gloss");
+                   printf(" %2d %%\n",pb);
+               break;
+            }
+            colIdx++;
+            i += 2;
+         }
+      }
+#endif
 #if 0
       else if ( strncmp(&buf[i], "WR:", 3 ) == 0 )
       {
@@ -863,8 +1087,8 @@ int decodeStatus(unsigned char *buf, int len, char *lang)
 
 int decodePrinterType(unsigned char *buf, int len)
 {
-   char *s = buf;
-   char *t = buf;
+   char *s = (char*)buf;
+   char *t = (char*)buf;
    int   i;
 
    if ( buf )
@@ -1021,55 +1245,79 @@ int alignHeads(char *lang, configData_t actConfig)
       command  = ALIGN_HEAD;
    }
 
-   for ( pass = 1; pass <= passes; pass++ )
+   if ( passes > 0 )
    {
-      /* print message 1 */
-      if ( pass == 1 )
+      for ( pass = 1; pass <= passes; pass++ )
       {
-         printText("\n\n%s\n",lang, "firstHeadAlign");
+         /* print message 1 */
+         if ( pass == 1 )
+         {
+            printText("\n\n%s\n",lang, "firstHeadAlign");
+         }
+         else
+         {
+            printText("\n\n%s\n",lang, "nextHeadAlign");
+         }
+ 
+         for(;;)
+         {
+            printText("%s",lang, "continue");
+            if ( askYn(lang,"yesorno") ) break;
+         }
+ 
+         /* and send query for pattern */
+         ret = callPrg(command, actConfig.dev, actConfig.align, pass, 0, NULL, function);
+         if ( ret == -1 )
+         {
+            fprintText(stderr,"%s\n",lang,"noOPen");
+            return 0;
+         }
+         /* ask for pattern # */
+         printText("\n%s\n",lang, color ? "chooseCPattern" : "choosePattern");
+         patternNo = askNo(1,choices);
+ 
+         /* save the pattern number */
+         ret = callPrg(command, actConfig.dev, actConfig.align, pass, patternNo, NULL, function);
       }
-      else
-      {
-         printText("\n\n%s\n",lang, "nextHeadAlign");
-      }
-      
+      /* print pattern 1 to n again */
+      printText("\n\n%s\n",lang, "nextHeadAlign");
       for(;;)
       {
          printText("%s",lang, "continue");
          if ( askYn(lang,"yesorno") ) break;
       }
-         
-      /* and send query for pattern */
-      ret = callPrg(command, actConfig.dev, actConfig.align, pass, 0, NULL, function);
+      ret = callPrg(command, actConfig.dev, actConfig.align, actConfig.passes, -1, NULL, function);
+ 
+      printText("\n%s",lang, "lastHeadAlign");
+   }
+   else if ( passes < 0 )
+   {
+      /* n passes printed all on the same time, choice = 0 -> print pattern */
+      ret = callPrg(command, actConfig.dev, actConfig.align, passes, 0, NULL, function);
       if ( ret == -1 )
       {
          fprintText(stderr,"%s\n",lang,"noOPen");
          return 0;
       }
-      /* ask for pattern # */
-      printText("\n%s\n",lang, color ? "chooseCPattern" : "choosePattern");
-      patternNo = askNo(1,choices);
-      
-      /* save the pattern number */
-      ret = callPrg(command, actConfig.dev, actConfig.align, pass, patternNo, NULL, function);
+      patternNo = 0;
+      passes    = -passes;
+      for ( pass = 0; pass < passes; pass++ )
+      {
+         /* ask for pattern # */
+         printText("\n%s", lang, "choosePattern2");
+         printf("%d.\n ",pass+1);
+         patternNo |= askNo(1,choices) << (8*pass);
+         /* save the pattern number */
+      }
+      /* choice > 0 -> set correction */
+      ret = callPrg(command, actConfig.dev, actConfig.align, -passes, patternNo, NULL, function);
+      printText("\n%s",lang, "lastHeadAlign");
    }
    
-   
-   /* print pattern 1 to n again */
-   printText("\n\n%s\n",lang, "nextHeadAlign");
-   for(;;)
-   {
-      printText("%s",lang, "continue");
-      if ( askYn(lang,"yesorno") ) break;
-   }
-   ret = callPrg(command, actConfig.dev, actConfig.align, actConfig.passes, -1, NULL, function);
-  
-   printText("\n%s",lang, "lastHeadAlign");
-      
    /* ask for saving into printer */
    if ( askYn(lang,"saveCancel") )
    {
-      /* send save into printer if applicable */
+      /* send save into printer if applicable, choice = 0  and pass > max pass */
       ret = callPrg(command,actConfig.dev, actConfig.align, 4 , 0 , NULL, function);
    }
    callPrg(RELEASE,actConfig.dev,0,0,0,NULL,NULL);
@@ -1077,6 +1325,7 @@ int alignHeads(char *lang, configData_t actConfig)
    return 1;
 }
 
+
 /*******************************************************************/
 /* Function exchangeInk                                            */
 /*                                                                 */
