@@ -34,14 +34,17 @@
 #include <fcntl.h>
 #include <sys/time.h>
 #include <unistd.h>
-#include <sys/poll.h>
 #include <signal.h>
 #include <errno.h>
 #include <sys/ioctl.h>
 #include <string.h>
 #include <sys/ipc.h>
 #include <sys/shm.h>
-
+#ifdef MACOS
+#include "fakepoll.h"
+#else
+#include <sys/poll.h>
+#endif
 #include <dirent.h>
 #include <sys/ioctl.h>
 
@@ -322,7 +325,7 @@ int getServerState(char *fifo)
 /*                                                                 */
 /*******************************************************************/
 
-static int decodePrinterType(unsigned char *buf, int len)
+static int decodePrinterType(char *buf, int len)
 {
    char *s = buf;
    char *t = buf;
@@ -689,7 +692,7 @@ int checkPrinter(char *dir, char *dev, char *type, char **device)
    }
    argp[length] = '\0';
 
-   if (checkPrinterDes(argp+1, type) == 1)
+   if (checkPrinterDes((char*)argp+1, type) == 1)
    {
       *device = fn;
       return 1;
@@ -1220,7 +1223,7 @@ int main(int argc, char **argv)
          shmem->block=2;
          /* try to detect the printer model */
          errno = 0;
-#if 0 /* this may cause problems if zhe printer has low ink */
+#if 0 /* this may cause problems if the printer has low ink */
          shmem->mode      = NO_PROT;
          shmem->command   = TEST_DEV;
 #else
@@ -1531,7 +1534,7 @@ int main(int argc, char **argv)
                /* print job detected, initialize print environment */
                terminate  = 0;
                stop       = 0;
-               sndSize    = writeD4Data(fd,"",0,0);
+               sndSize    = writeD4Data(fd,(uc*)"",0,0);
                sockId     = shmem->dataSocketId;
                if ( sndSize > 0 )
                {
